"""
Making requests from the Federal Register API.
"""

from .get_documents import (
    BASE_URL,
    BASE_PARAMS,
    DEFAULT_FIELDS, 
    QueryError,
    InputFileError, 
    get_documents_by_date, 
    get_documents_by_number, 
    parse_document_numbers, 
    _retrieve_results_by_next_page
)

__all__ = [
    "BASE_URL",
    "BASE_PARAMS",
    "DEFAULT_FIELDS",
    "QueryError",
    "InputFileError",
    "get_documents_by_date", 
    "get_documents_by_number", 
    "parse_document_numbers", 
    ]
